from __future__ import print_function
import Cmm
import numpy
import os
import Manyo as mm
import Manyo.Utsusemi as mu
import math

###############################################################################
# Efficiency Correction Util
###############################################################################

def PutIntegralIntensityArray(DAT,energyMin,energyMax):
    retS=[]
    retM=[]
    EC = DAT.Put(0).Put(0)
    ecXn = numpy.array(EC.PutXList())
    ecX  = (ecXn[1:]+ecXn[:-1])*0.5
    minReginArray  = ecX>energyMin
    maxReginArray  = ecX<energyMax
    regionArray    = numpy.logical_and(minReginArray,maxReginArray)

    ecaSize = DAT.PutTableSize()
    for i in range(ecaSize):
        ECA = DAT.Put(i)
        retAS=[]
        retAM=[]
        ecSize = ECA.PutTableSize()
        for j in range(ecSize):
            EC = ECA.Put(j)
            mask = EC.PutHeaderPointer().PutInt4("MASKED")
            #mask==1 masked / mask ==0 not masked

            ecY  = numpy.array(EC.PutYList())
            ecE  = numpy.array(EC.PutEList())
            errMaskArray   = ecE>=0
            conditionArray = numpy.logical_and(regionArray,errMaskArray)
            ecY            = ecY[conditionArray]

            sumVal = numpy.sum(ecY)

            retAS.append(sumVal)
            retAM.append(mask)
        retS.append(retAS)
        retM.append(retAM)
    return retS,retM
#------------------------------------------------------------------------------
def PutIntegralIntensityArray2(DAT,energyMin,energyMax):
    retS=[]
    retM=[]
    EC = DAT.Put(0).Put(0)
    ecXn = numpy.array(EC.PutXList())
    ecX  = (ecXn[1:]+ecXn[:-1])*0.5
    minReginArray  = ecX>energyMin
    maxReginArray  = ecX<energyMax
    regionArray    = numpy.logical_and(minReginArray,maxReginArray)

    ecaSize = DAT.PutTableSize()
    for i in range(ecaSize):
        ECA = DAT.Put(i)
        retAS=[]
        retAM=[]
        ecSize = ECA.PutTableSize()
        for j in range(ecSize):
            EC = ECA.Put(j)
            mask = EC.PutHeaderPointer().PutInt4("MASKED")
            #mask==1 masked / mask ==0 not masked

            ecY  = numpy.array(EC.PutYList())
            ecE  = numpy.array(EC.PutEList())
            errMaskArray   = ecE>=0
            conditionArray = numpy.logical_and(regionArray,errMaskArray)
            ecY            = ecY[conditionArray]
            if mask==0:
                sumVal = numpy.sum(ecY)
            else:
                sumVal = 0
            retAS.append(sumVal)
            retAM.append(mask)
        retS.append(retAS)
        retM.append(retAM)
    return retS,retM
#------------------------------------------------------------------------------
def PutAverageIntegralIntensity(integralArray,maskArray):
    integralArray = numpy.array(integralArray)
    maskArray = numpy.array(maskArray)

    cond = (maskArray == 0)
    integralArray = integralArray[cond]

    #average = numpy.average(integralArray)
    sumval  = numpy.sum(integralArray)
    sizeval = numpy.size(integralArray)

    #print "average:",average
    print("sumval:",sumval)
    print("sizeval:",sizeval)
    print("average:",sumval/sizeval)

    return sumval/sizeval
#------------------------------------------------------------------------------
def PutEbinWidth(DAT):
    EC  = DAT.Put(0).Put(0)
    ECX = EC.PutXList()
    d   = ECX[1]-ECX[0]
    print("Ebin Width :",d)
    return d
#------------------------------------------------------------------------------
def WriteMaskFileFromAverageIntensity(output,integralArray,maskArray,averageVal,thresh=0.5):
    f = open(output,"w")

    #fPath=os.path.join(os.environ["UTSUSEMI_USR_DIR"],"ana","xml",output)
    fPath=os.path.join(mu.UtsusemiEnvGetBaseDir(),mu.UtsusemiEnvGetInstCode(),"ana","xml",output)
    g = open(fPath,"w")

    integralArray = numpy.array(integralArray)
    maskArray     = numpy.array(maskArray)
    condMask      = (maskArray == 0)
    condHalfVal   = (integralArray >= averageVal*thresh)

    cond = numpy.logical_and(condHalfVal,condMask)
    sep = " "
    for i,line in enumerate(cond):
        if numpy.sum(line)==0:
            text = str(i)+"\n"
        else:
            vals = ["%d.%d"%(i,j) for j,flag in enumerate(line) if not flag]
            text = sep.join(vals)+"\n"
        f.write(text)
        g.write(text)
    f.close()
    g.close()
    print(fPath)
#------------------------------------------------------------------------------
def CreateMaskFileFromAverageIntensity(DAT,energyMin,energyMax,output,thresh=0.5):
    integs,masks = PutIntegralIntensityArray(DAT,energyMin,energyMax)
    average      = PutAverageIntegralIntensity(integs,masks)
    WriteMaskFileFromAverageIntensity(output,integs,masks,average,thresh)
#------------------------------------------------------------------------------
def PutCrossSectionScaleFactor(DAT, energyMin, energyMax,
                               container1, sample_filename1,
                               nist_filename):
    integs,masks = PutIntegralIntensityArray(DAT,energyMin,energyMax)
    average      = PutAverageIntegralIntensity(integs,masks)
    c1 = GetCrossSection(container1, sample_filename1, nist_filename)
    d  = PutEbinWidth(DAT)
    return average*d/c1
#------------------------------------------------------------------------------
def DoCrossSectionScaleFactorCorrection(ECM,scalefactor,sampleMass=1.0,unitMass=1.0,mode=1):
    AvogadroC =6.022141*1e23
    #d = PutEbinWidth(ECM)
    d = 1.0  # changed by M. Matsuura 2015.4.31
    ecaSize = ECM.PutTableSize()
    ecm = mm.ElementContainerMatrix(ECM.PutHeader())
    for i in range(ecaSize):
        ECA = ECM.Put(i)
        eca = mm.ElementContainerArray(ECA.PutHeader())
        ecSize = ECA.PutTableSize()
        for j in range(ecSize):
            EC = ECA.Put(j)
            r = d / scalefactor / (sampleMass/unitMass) / AvogadroC
            if mode==1:
                ec = EC.Mul(r)
            else:
                if r!=0:
                    ec = EC.Mul(1.0/r)
                else:
                    ec = EC.Mul(0.0)
            ech=ec.PutHeaderPointer()

            ecY  = numpy.array(ec.PutYList())
            ecE  = numpy.array(ec.PutEList())
            errMaskArray   = ecE>=0
            ecY            = ecY[errMaskArray]
            sumVal = numpy.sum(ecY)

            ech.OverWrite("TotalCounts",sumVal)
            eca.Add(ec)
        ecm.Add(eca)
    return ecm
#------------------------------------------------------------------------------
def PutAverage(DAT, energyMin, energyMax):
    integs,masks = PutIntegralIntensityArray(DAT,energyMin,energyMax)
    average      = PutAverageIntegralIntensity(integs,masks)
    return average
#------------------------------------------------------------------------------
def WriteEfficiencyFile(output,integralArray,maskArray,averageVal):
    f = open(output,"w")

    #fPath=os.path.join(os.environ["HOME"],"ana","Efficiency",output)
    fPath=os.path.join(mu.UtsusemiEnvGetUserDir(),"ana","DNA","Efficiency",output)
    g = open(fPath,"w")

    integralArray = numpy.array(integralArray)
    psdSize, pixelSize = numpy.shape(integralArray)

    sep=","
    vals = integralArray/averageVal
    for i in range(psdSize):
        val = vals[i]
        valStr = [ "%.4e"%v for v in val]
        text = sep.join(valStr) +"\n"
        f.write(text)
        g.write(text)

    g.close()
    f.close()
#------------------------------------------------------------------------------
def CreateEfficiencyFile(DAT,energyMin,energyMax,output):
    integs,masks = PutIntegralIntensityArray(DAT,energyMin,energyMax)
    average      = PutAverageIntegralIntensity(integs,masks)
    WriteEfficiencyFile(output,integs,masks,average)
#------------------------------------------------------------------------------
def ReadEfficiencyFile(filename):
    f=open(filename)
    sep=","

    v=[]
    for line in f:
        ll = line.split(sep)
        p=[float(val) for val in ll]

        v.append(p)
    return v
#------------------------------------------------------------------------------
def ExecEfficiencyCorrection(ECM,d,mode=1):

    ecaSize = ECM.PutTableSize()
    ecm = mm.ElementContainerMatrix(ECM.PutHeader())
    for i in range(ecaSize):
        ECA = ECM.Put(i)
        eca = mm.ElementContainerArray(ECA.PutHeader())
        ecSize = ECA.PutTableSize()
        for j in range(ecSize):
            EC = ECA.Put(j)
            if mode==1:
                ec = EC.Mul(d[i][j])
            else:
                r=d[i][j]
                if r!=0:
                    ec = EC.Mul(1.0/d[i][j])
                else:
                    #ec = EC.Mul(0.0)[20251030 TY] To keep sign of error it is changed
                    ec = EC.Mul(1.0) 
            ech=ec.PutHeaderPointer()

            ecY  = numpy.array(ec.PutYList())
            ecE  = numpy.array(ec.PutEList())
            errMaskArray   = ecE>=0
            ecY            = ecY[errMaskArray]
            sumVal = numpy.sum(ecY)

            ech.OverWrite("TotalCounts",sumVal)
            eca.Add(ec)
        ecm.Add(eca)
    return ecm
#------------------------------------------------------------------------------
def DoEfficiencyCorrection(DAT,filename,mode=1):
    d = ReadEfficiencyFile(filename)
    DAT2 = ExecEfficiencyCorrection(DAT,d,mode)
    return DAT2

###############################################################################
# Cross Section Utils (from sample data file)
###############################################################################

def _DataExtract(filename,container):

    f = open(filename, 'r')
    dataIn = f.read()
    f.close()

    if container == "cell":
        ini = dataIn.find("// wall")
        oth = dataIn.find("// body")
    else:
        ini = dataIn.find("// body")
        oth = dataIn.find("// wall")

    if ini < oth:
        dataOut=dataIn[ini:oth]
    else:
        dataOut=dataIn[ini:]

    return dataOut

#------------------------------------------------------------------------------

def _GetData(data,tag,ind=0):

    h_i = data.find(tag,ind)

    if h_i<0:
        return -1
    else:

        h_c = data.rfind("//",0,h_i)
        h_n = data.rfind("\n",0,h_i)
        if h_c > h_n:
            return _GetData(data,tag,h_i+1)


        if data[h_i-1]==" " or data[h_i-1]=="\n" or h_i==0:

            h_e = data.find("\n",h_i)
            if  "//" in data[h_i:h_e]:
                h_e = data[h_i:h_e].find("//")+h_i

            if h_i > h_e:
                return -1
            else:
                h = float(data[h_i:h_e].split("=")[-1])
                return h
        else:
            return _GetData(data,tag,h_i+1)
#------------------------------------------------------------------------------

def _GetShape(data,ind=0):

    form_i= data.find("form",ind)

    form_c = data.rfind("//",0,form_i)
    form_n = data.rfind("\n",0,form_i)
    if form_c > form_n:
        return _GetShape(data,form_i+1)

    form_e= data.find("\n",form_i)

    if "cylinder" in data[form_i:form_e]:
        return "cylinder"
    elif "column"  in data[form_i:form_e]:
        return "column"
    else:
        return ""

#------------------------------------------------------------------------------

def _GetAtomicInfo(data):

    # Get Atomic Info Index
    atom_ds = []

    tmp_i=-1
    while 1:
        tmp_i = data.find("\n",tmp_i+1)

        tmp_c=data.find("//",  tmp_i+1)
        tmp_i=data.find("atom",tmp_i+1)
        if tmp_i < 0:
            break
 
        if tmp_c > 0 and tmp_c < tmp_i:
            pass
        else:
            tmp_e = data.find("\n",tmp_i+1)
            if tmp_c > 0 and tmp_c < tmp_e:
                tmp_e = tmp_c 

            atom_ds.append(data[tmp_i:tmp_e])

    # Get Atomic Info Data
    atoms = []
    ratio_all=0.0
    for atom_d in atom_ds:

        a1 = atom_d.find("=")+1
        a2 = atom_d.find(",")
        name = atom_d[a1:a2].strip()

        a3 = atom_d.find("=",a2)+1
        a4 = -1
        try:
            ratio = float(atom_d[a3:a4].strip())
            atoms.append([name,ratio])
            ratio_all += ratio
        except:
            pass

        if "mol" in atom_d:
            mode = "mol"
        else:
            mode = "wt"

    # re - normalize of atomic ratio
    for i in range(len(atoms)):
        atoms[i][1] = atoms[i][1]/ratio_all*100

    return atoms,mode

#------------------------------------------------------------------------------

def _ReadNISTData(atoms,nist_filename):

    g = open(nist_filename)
    data2 = g.read()
    g.close()

    for i in range(len(atoms)):
        b1=data2.find(atoms[i][0])
        if b1<0:
            atoms[i] += [0.0,0.0,0.0,0.0,0.0,0.0]
            print("\n" + atoms[i][0] + " cross section data is not found.")
        else:
            b2=data2.find("\n",b1)
            tmp = data2[b1:b2].split()
            tmp = [ float(a) for a in tmp[1:7]]
            atoms[i] += tmp

    return atoms

#------------------------------------------------------------------------------

def _GetTotalCrossSection(atoms,mode,M_tot):
    AvogadroC =6.022141*1e23

    print("\natoms:")
    if mode == "mol":
        mean_atomic_M = 0.0
        for i in range(len(atoms)):
            ratio     = atoms[i][1]/100.0
            atomic_M  = atoms[i][2]
            mean_atomic_M += atomic_M * ratio

        N_tot = M_tot / mean_atomic_M * AvogadroC # numer of atoms

        for i in range(len(atoms)):
            ratio    = atoms[i][1]/100.0
            CSinc    = atoms[i][5]
            atoms[i].append(ratio * N_tot * CSinc / (4.0*math.pi))
            print(atoms[i][0],":",mode,"%",atoms[i][1],":",CSinc,"[barn]")
    else: # mode == "wt"
        for i in range(len(atoms)):
            ratio    = atoms[i][1]/100.0
            M_each   = M_tot*ratio
            atomic_M = atoms[i][2]
            if atomic_M==0:
                N_each=0
            else:
                N_each   = M_each / atomic_M *AvogadroC  # numer of atoms
            CSinc    = atoms[i][5]
            atoms[i].append(N_each * CSinc / (4.0*math.pi))
            print(atoms[i][0],":",mode,"%",atoms[i][1],":",CSinc,"[barn]")
    sumCS=0.0
    for i in range(len(atoms)):
        sumCS += atoms[i][8]

    return sumCS

#------------------------------------------------------------------------------

def _GetTotalUnitCell(atoms,mode,M_tot):
    AvogadroC =6.022141*1e23

    print("\natoms:")
    if mode == "mol":
        mean_atomic_M = 0.0
        for i in range(len(atoms)):
            ratio     = atoms[i][1]/100.0
            atomic_M  = atoms[i][2]
            mean_atomic_M += atomic_M * ratio

        N_tot = M_tot / mean_atomic_M * AvogadroC # numer of atoms
    else: # mode == "wt"
        N_tot=0
        for i in range(len(atoms)):
            ratio    = atoms[i][1]/100.0
            M_each   = M_tot*ratio
            atomic_M = atoms[i][2]
            if atomic_M==0:
                N_each=0
            else:
                N_each   = M_each / atomic_M *AvogadroC  # numer of atoms
            N_tot += N_each

    return N_tot

#------------------------------------------------------------------------------

def GetCrossSection(container, sample_filename, nist_filename):
    data = _DataExtract(sample_filename, container)

    wpv = _GetData(data,"weight_per_volume") * 1000.0 # g/m^3

    shape = _GetShape(data)
    if shape=="cylinder":
        r_l = _GetData(data,"r_large") # mm
        r_s = _GetData(data,"r_small") # mm
    elif shape=="column":
        r_l = _GetData(data,"r_large") # mm
        r_s = 0.0                     # mm

    h  = _GetData(data,"height")       # mm
    bs = _GetData(data,"beam_size")    # mm
    vu = _GetData(data,"volume")       # mm^3

    if vu < 0:
        if bs < 0:
            volume = math.pi * (r_l * r_l - r_s * r_s) * h * 1e-9  # m^3
            print("\nsample volume = pi * (r_l * r_l - r_s * r_s) * h [mm ^ 3]")
            print("          r_l =",r_l)
            print("          r_s =",r_s)
            print("            h =",h)
        else:
            volume = math.pi * (r_l * r_l - r_s * r_s) * bs * 1e-9 # m^3
            print("\nsample volume = pi * (r_l * r_l - r_s * r_s) * beam_size [mm ^ 3]")
            print("          r_l =",r_l)
            print("          r_s =",r_s)
            print("    beam_size =",bs)

    else:
            volume = vu * 1e-9                                     # m^3
            print("\nsample volume = volume [mm ^ 3]")
            print("       volume =",vu)

    m_tot      = wpv *volume                                       # g

    atoms,mode = _GetAtomicInfo(data)

    atoms      = _ReadNISTData(atoms,nist_filename)

    cs         = _GetTotalCrossSection(atoms,mode,m_tot)

    return cs

#------------------------------------------------------------------------------

def GetNumOfUnitCell(container, sample_filename, nist_filename):
    data = _DataExtract(sample_filename, container)

    wpv = _GetData(data,"weight_per_volume") * 1000.0 # g/m^3

    shape = _GetShape(data)
    if shape=="cylinder":
        r_l = _GetData(data,"r_large") # mm
        r_s = _GetData(data,"r_small") # mm
    elif shape=="column":
        r_l = _GetData(data,"r_large") # mm
        r_s = 0.0                     # mm

    h  = _GetData(data,"height")       # mm
    bs = _GetData(data,"beam_size")    # mm
    vu = _GetData(data,"volume")       # mm^3

    if vu < 0:
        if bs < 0:
            volume = math.pi * (r_l * r_l - r_s * r_s) * h * 1e-9  # m^3
            print("\nsample volume = pi * (r_l * r_l - r_s * r_s) * h [mm ^ 3]")
            print("          r_l =",r_l)
            print("          r_s =",r_s)
            print("            h =",h)
        else:
            volume = math.pi * (r_l * r_l - r_s * r_s) * bs * 1e-9 # m^3
            print("\nsample volume = pi * (r_l * r_l - r_s * r_s) * beam_size [mm ^ 3]")
            print("          r_l =",r_l)
            print("          r_s =",r_s)
            print("    beam_size =",bs)

    else:
            volume = vu * 1e-9                                     # m^3
            print("\nsample volume = volume [mm ^ 3]")
            print("       volume =",vu)

    m_tot      = wpv *volume                                       # g

    atoms,mode = _GetAtomicInfo(data)

    atoms      = _ReadNISTData(atoms,nist_filename)

    N_tot = _GetTotalUnitCell(atoms,mode,m_tot)
    return N_tot

#------------------------------------------------------------------------------
# added by M. Matsuura 2015.6.8
def DeleteMask_DirectBeam(input, output):
    f = open(input, 'r')
    dat = f.readlines()
    f.close

    i=0
    for psd in dat:
        if i==167:
            print(i)
            psdmask = dat[i].split(" ")
            j=0
            psd167_1st=0
            psd167_2nd=0
            for pixel in psdmask:
                [psdno,pxlno] = psdmask[j].split(".")
                if int(pxlno)!=j:
                    psd167_2nd=int(pxlno)
                    break
                psd167_1st=j
                j+=1
            print(psd167_1st, psd167_2nd)
        if i==168:
            print(i)
            psdmask = dat[i].split(" ")
            j=0
            psd168_1st=0
            psd168_2nd=0
            for pixel in psdmask:
                [psdno,pxlno] = psdmask[j].split(".")
                if int(pxlno)!=j:
                    psd168_2nd=int(pxlno)
                    break
                psd168_1st=j
                j+=1
            print(psd168_1st, psd168_2nd)
        if i==169:
            print(i)
            psdmask = dat[i].split(" ")
            j=0
            psd169_1st=0
            psd169_2nd=0
            for pixel in psdmask:
                [psdno,pxlno] = psdmask[j].split(".")
                if int(pxlno)!=j:
                    psd169_2nd=int(pxlno)
                    break
                psd169_1st=j
                j+=1
            print(psd169_1st, psd169_2nd)
        i+=1
    # make new mask data for psd167
    psd167txt=""
    i=0
    while i < psd167_1st+1: 
        psd167txt+="167."+str(i)+" "
        i+=1
    i=psd167_2nd
    while i <119:
        psd167txt+="167."+str(i)+" "
        i+=1
    psd167txt+="167.119\n"
    dat[167]=psd167txt
    
    # make new mask data for psd168
    psd168txt=""
    i=0
    
    while i < psd168_1st+1: 
        psd168txt+="168."+str(i)+" "
        i+=1
    
    i=psd168_2nd
    while i <119:
        psd168txt+="168."+str(i)+" "
        i+=1
    
    psd168txt+="168.119\n"
    dat[168]=psd168txt
    
    # make new mask data for psd169
    psd169txt=""
    i=0
    
    while i < psd169_1st+1: 
        psd169txt+="169."+str(i)+" "
        i+=1
    
    i=psd169_2nd
    while i <119:
        psd169txt+="169."+str(i)+" "
        i+=1
    
    psd169txt+="169.119\n"
    dat[169]=psd169txt
    
    g = open(output, 'w')
    i=0
    for psd in dat:
        g.writelines(dat[i])
        print(dat[i])
        i+=1
    g.close
#------------------------------------------------------------------------------
def DeleteMask_Spurious(input, output):
    f = open(input, 'r')
    dat = f.readlines()
    f.close

    i=0
    for psd in dat:
        if i==49:
            print(i)
            psdmask = dat[i].split(" ")
            j=0
            psd49=0
            for pixel in psdmask:
                [psdno,pxlno] = psdmask[j].split(".")
                if int(pxlno)!=j:
                    psd49=j-1
                    break
                j+=1
            print(psd49)
        if i==50:
            print(i)
            psdmask = dat[i].split(" ")
            j=0
            psd50=0
            for pixel in psdmask:
                [psdno,pxlno] = psdmask[j].split(".")
                if int(pxlno)!=j:
                    psd50=j-1
                    break
                j+=1
            print(psd50)
        if i==51:
            print(i)
            psdmask = dat[i].split(" ")
            j=0
            psd51=0
            for pixel in psdmask:
                [psdno,pxlno] = psdmask[j].split(".")
                if int(pxlno)!=j:
                    psd51=j-1
                    break
                j+=1
            print(psd51)
        if i==61:
            print(i)
            psdmask = dat[i].split(" ")
            j=0
            psd61=0
            for pixel in psdmask:
                [psdno,pxlno] = psdmask[j].split(".")
                if int(pxlno)!=j:
                    psd61=j-1
                    break
                j+=1
            print(psd61)
        i+=1

    # make new mask 
    # psd49
    psd49txt=""
    i=0
    while i < psd49+1: 
        psd49txt+="49."+str(i)+" "
        i+=1
    i=44
    while i <119:
        psd49txt+="49."+str(i)+" "
        i+=1
    
    psd49txt+="49.119\n"
    dat[49]=psd49txt
    
    # psd50
    psd50txt=""
    i=0
    while i < psd50+1: 
        psd50txt+="50."+str(i)+" "
        i+=1
    i=44
    while i <119:
        psd50txt+="50."+str(i)+" "
        i+=1
    psd50txt+="50.119\n"
    dat[50]=psd50txt
    
    # psd51
    psd51txt=""
    i=0
    while i < psd51+1: 
        psd51txt+="51."+str(i)+" "
        i+=1
    i=44
    while i <119:
        psd51txt+="51."+str(i)+" "
        i+=1
    psd51txt+="51.119\n"
    dat[51]=psd51txt
    
    # psd61
    psd61txt=""
    i=0
    while i < psd61+1: 
        psd61txt+="61."+str(i)+" "
        i+=1
    i=44
    while i <119:
        psd61txt+="61."+str(i)+" "
        i+=1
    psd61txt+="61.119\n"
    dat[61]=psd61txt
    
    g = open(output, 'w')
    i=0
    for psd in dat:
        g.writelines(dat[i])
        print(dat[i])
        i+=1
    g.close



#!/usr/bin/python3
# -*- coding: utf-8 -*-

import Cmm
import Manyo
import Manyo.MLF as mm

def RadialCollimatorCorrect(dat,vdat,integRange="All",pixelRange="All"):
    """
    Data Correction for radial collimator on SIK
    
    @param dat[Def:DAT] (ElementContainerMatrix) Target data
    @param vdat[Def:DAT] (ElementContainerMatrix) Vanadium data
    @param integRange (string) Integrate range "All" or "<start>:<end>"
    @param pixelRange (string) Pixel range to be used "All" or "<start>:<end>"
    @retval None
    """
    if dat.PutSize()!=vdat.PutSize():
        raise UserWarning("ERROR : Data sizes are different between dat and vdat.")

    v_integRange = integRange.split(":")
    isAll = True
    x_range = []
    if len(v_integRange)==2:
        try:
            x_range = [float(v_integRange[0]), float(v_integRange[1])]
            isAll = False
        except:
            raise UserWarning("ERROR : invalid integRange argument")
    elif integRange.upper()!="ALL":
        raise UserWarning("ERROR : invalid integRange argument")

    start_pix = 0
    end_pix = dat(0).PutSize()-1
    
    if pixelRange.upper()!="ALL":
        v_pixel = pixelRange.split(":")
        if len(v_pixel)!=2:
            raise UserWarning("ERROR : invalid pixelRange argument")
        start_pix = int(v_pixel[0])
        end_pix = int(v_pixel[1])
        if start_pix>end_pix:
            tmp = start_pix
            start_pix = end_pix
            end_pix = tmp
        if end_pix>=dat(0).PutSize():
            end_pix = dat(0).PutSize()-1
    

    sum_vals = 0.0
    list_vals = []
    
    ignore_num =0
    for i in range(vdat.PutSize()):
        num_of_ec = 0
        det_v = Manyo.MakeUInt4Vector()
        pix_v = Manyo.MakeUInt4Vector()
        for j in range(start_pix,end_pix+1):
            if vdat(i,j).PutHeaderPointer().PutInt4("MASKED")==0:
                det_v.append(i)
                pix_v.append(j)
                num_of_ec += 1
        tt=mm.AverageElementContainerMatrix(vdat,det_v,pix_v)
        ec = tt.GetSum()
        if isAll:
            val = ec.Sum()
        else:
            p = ec.Sum(x_range[0],x_range[1])
            val = p.first
        if val==0.0 or num_of_ec==0:
            ignore_num +=1
        else:
            val /= float(num_of_ec)
            sum_vals += val
        list_vals.append(val)

    ave_vals = sum_vals/float(vdat.PutSize()-ignore_num)
    for i,val in enumerate(list_vals):
        if val!=0:
            dat(i).MulMySelf(1.0/val*ave_vals)




#! /usr/bin/env python
# -*- coding: utf-8 -*-

import os
import numpy as np
import scipy.optimize
import Manyo.SAS as ms
import Manyo as mm
import math
import utsusemi.SAS.ana.Reduction.HistBaseCommands as HBC
import utsusemi.SAS.ana.Reduction.CorrBaseCommands as CBC
"""
Base Functions for SAS
Trans.py
- GetTransNormByMonCounts
- GetTransNormByMonCountsTS
- GetTransNormByMon
- GetTransNormByMonTS
"""

#########################################
def GetTransNormByMonCounts(SmplRunNo="104000", DrctRunNo="104000", AxType="lambda", XRange="0.7 7.6 0.1", frameBoundary=0.0, CalibEffi=True, paramFiles="- -" ):
    """
    Monitor histogram of sample and direct followed by a transmittance calculation.
    @param SmplRunNo     (string)    Sample measurement run number
    @param DrctRunNo     (string)    Direct beam measurement run number
    @param AxType        (string)  Type must be "tof', "lambda" or "lamda2"
    @param XRange        (string) "startX endX widthX" separated with space (not comma)
    @param frameBoundary (float)
    @param CalibEffi     (bool)    Monitor-detector efficiency calibration
    @param paramFiles    (string) "<wiringInfo> <detectorInfo>[ <caseInfo>]"
    @retval list of Tr
    """
    import Com
    s = XRange.split()
    norm_range = "%s %s"%(s[0], s[1])

    smpl1s = HBC.GetMonHistSAS(SmplRunNo, 1, AxType, XRange, 0, "-1 -1", str(frameBoundary), CalibEffi, paramFiles=paramFiles)
    smpl2s = HBC.GetMonHistSAS(SmplRunNo, 2, AxType, XRange, 0, "-1 -1", str(frameBoundary), CalibEffi, paramFiles=paramFiles)
    if type(smpl1s)!=list:
        smpl1s = [smpl1s]
        smpl2s = [smpl2s]
    for smpl1, smpl2 in zip(smpl1s,smpl2s):
        CBC.NormByMonCounts(smpl2, smpl1, "Lamb", norm_range, 1.0)

    drct1 = HBC.GetMonHistSAS(DrctRunNo, 1, AxType, XRange, 0, "-1 -1", str(frameBoundary), CalibEffi)
    drct2 = HBC.GetMonHistSAS(DrctRunNo, 2, AxType, XRange, 0, "-1 -1", str(frameBoundary), CalibEffi)
    CBC.NormByMonCounts(drct2, drct1, "Lamb", norm_range, 1.0)

    ret = []
    for smpl2 in smpl2s:
        ret.append( Com.ExecFunctionCode("A/B", smpl2, drct2) )
    if len(ret)==1:
        return ret[0]
    return ret

#########################################
def GetTransNormByMonCountsTS(SmplRunNo="104000", DrctRunNo="104000", AxType="lambda", XRange="0.7 7.6 0.1", frameBoundary=0.0, CalibEffi=True, SmplTSlice="-1 -1", DrctTSlice="-1 -1", paramFiles="- -"):
    """
    Monitor histogram of sample and direct followed by a transmittance calculation.
    @param SmplRunNo     (string)    Sample measurement run number
    @param DrctRunNo     (string)    Direct beam measurement run number
    @param AxType        (string)  Type must be "tof', "lambda" or "lamda2"
    @param XRange        (string) "startX endX widthX" separated with space (not comma)
    @param frameBoundary (float)
    @param CalibEffi     (bool)    Monitor-detector efficiency calibration
    @param SmplTSlice    (string) Time Slice parameters for Sample run
    @param DrctTSlice    (string) Time Slice parameters for Directrun
    @param paramFiles    (string) "<wiringInfo> <detectorInfo>[ <caseInfo>]
    @retval Tr
    """
    import Com
    s = XRange.split()
    norm_range = "%s %s"%(s[0], s[1])

    smpl1s = HBC.GetMonHistSAS(SmplRunNo, 1, AxType, XRange, 0, SmplTSlice, str(frameBoundary), CalibEffi, paramFiles=paramFiles)
    smpl2s = HBC.GetMonHistSAS(SmplRunNo, 2, AxType, XRange, 0, SmplTSlice, str(frameBoundary), CalibEffi, paramFiles=paramFiles)
    if type(smpl1s)!=list:
        smpl1s = [smpl1s]
        smpl2s = [smpl2s]
    for smpl1,smpl2 in zip(smpl1s,smpl2s):
        CBC.NormByMonCounts(smpl2, smpl1, "Lamb", norm_range, 1.0)

    drct1 = HBC.GetMonHistSAS(DrctRunNo, 1, AxType, XRange, 0, DrctTSlice, str(frameBoundary), CalibEffi)
    drct2 = HBC.GetMonHistSAS(DrctRunNo, 2, AxType, XRange, 0, DrctTSlice, str(frameBoundary), CalibEffi)
    CBC.NormByMonCounts(drct2, drct1, "Lamb", norm_range, 1.0)

    ret = []
    for smpl2 in smpl2s:
        ret.append(Com.ExecFunctionCode("A/B", smpl2, drct2))
    if len(ret)==1:
        return ret[0]
    return ret

#########################################
def GetTransNormByMon(SmplRunNo="104000", DrctRunNo="104000", Xrange="0.7 7.6 0.1 dL", frameBoundary="0.0", CalibEffi=True, useMon="MON1", paramFiles="- -" ):
    """
    Monitor histogram of sample and direct followed by a transmittance calculation.
    @param SmplRunNo     (string)    Sample measurement run number
    @param DrctRunNo     (string)    Direct beam measurement run number
    @param Xrange        (string)  "startX endX widthX type" separated with space (not comma) : type=[dL, dL/L, tof]
    @param frameBoundary (float)
    @param CalibEffi     (bool)    Monitor-detector efficiency calibration
    @param useMon        (strng)  Upstream Monitor (MON1 or MON3)
    @param paramFiles    (string) "<wiringInfo> <detectorInfo>[ <caseInfo>]
    @retval Tr
    """
    import Com
    s = Xrange.split()
    norm_range = "%s %s"%(s[0], s[1])

    x_range = Xrange.split( " " )
    AxType=""
    if len(x_range)!=4:
        raise UserWarning("Xrange is invalid.")
    if x_range[3] == "dL":
        AxType = "lambda"
    elif x_range[3] == "dL/L":
        AxType = "lambda2"
    elif x_range[3] == "tof":
        AxType = "tof"
    else:
        raise UserWarning("Xrange is invalid.")
    XRange = "%s %s %s"%(x_range[0],x_range[1],x_range[2])

    monUpStream = 1
    if useMon=="MON1":
        monUpStream = 1
    elif useMon=="MON3":
        monUpStream = 3
    else:
        raise UserWarning("useMon is invalid.")

    smpl1s = HBC.GetMonHistSAS(SmplRunNo, monUpStream, AxType, XRange, 0, "-1 -1", frameBoundary, CalibEffi, paramFiles=paramFiles)
    smpl2s = HBC.GetMonHistSAS(SmplRunNo, 2, AxType, XRange, 0, "-1 -1", frameBoundary, CalibEffi, paramFiles=paramFiles)
    if type(smpl1s)!=list:
        smpl1s = [smpl1s]
        smpl2s = [smpl2s]
    for smpl1,smpl2 in zip(smpl1s,smpl2s):
        CBC.NormByMonCounts(smpl2, smpl1, "Lamb", norm_range, 1.0)

    drct1 = HBC.GetMonHistSAS(DrctRunNo, monUpStream, AxType, XRange, 0, "-1 -1", frameBoundary, CalibEffi)
    drct2 = HBC.GetMonHistSAS(DrctRunNo, 2, AxType, XRange, 0, "-1 -1", frameBoundary, CalibEffi)
    CBC.NormByMonCounts(drct2, drct1, "Lamb", norm_range, 1.0)

    ret = []
    for smpl2 in smpl2s:
        ret.append(Com.ExecFunctionCode("A/B", smpl2, drct2))
    if len(ret)==1:
        return ret[0]
    return ret

#########################################
def GetTransNormByMonTS(SmplRunNo="104000", DrctRunNo="104000", Xrange="0.7 7.6 0.1 dL", frameBoundary="0.0", CalibEffi=True, SmplTSlice="-1 -1", DrctTSlice="-1 -1", useMon="MON1", paramFiles="- -"):
    """
    Monitor histogram of sample and direct followed by a transmittance calculation.
    @param SmplRunNo     (string)    Sample measurement run number
    @param DrctRunNo     (string)    Direct beam measurement run number
    @param Xrange        (string)  "startX endX widthX type" separated with space (not comma) : type=[dL, dL/L, tof]
    @param frameBoundary (float)
    @param CalibEffi     (bool)    Monitor-detector efficiency calibration
    @param SmplTSlice    (string) Time Slice parameters for Sample run
    @param DrctTSlice    (string) Time Slice parameters for Directrun
    @param useMon        (strng)  Upstream Monitor (MON1 or MON3)
    @param paramFiles    (string) "<wiringInfo> <detectorInfo>[ <caseInfo>]
    @retval Tr
    """
    import Com
    s = Xrange.split()
    norm_range = "%s %s"%(s[0], s[1])

    x_range = Xrange.split( " " )
    AxType=""
    if len(x_range)!=4:
        raise UserWarning("Xrange is invalid.")
    if x_range[3] == "dL":
        AxType = "lambda"
    elif x_range[3] == "dL/L":
        AxType = "lambda2"
    elif x_range[3] == "tof":
        AxType = "tof"
    else:
        raise UserWarning("Xrange is invalid.")
    XRange = "%s %s %s"%(x_range[0],x_range[1],x_range[2])

    monUpStream = 1
    if useMon=="MON1":
        monUpStream = 1
    elif useMon=="MON3":
        monUpStream = 3
    else:
        raise UserWarning("useMon is invalid.")

    smpl1s = HBC.GetMonHistSAS(SmplRunNo, monUpStream, AxType, XRange, 0, SmplTSlice, frameBoundary, CalibEffi, paramFiles=paramFiles)
    smpl2s = HBC.GetMonHistSAS(SmplRunNo, 2, AxType, XRange, 0, SmplTSlice, frameBoundary, CalibEffi, paramFiles=paramFiles)
    if type(smpl1s)!=list:
        smpl1s = [smpl1s]
        smpl2s = [smpl2s]
    for smpl1,smpl2 in zip(smpl1s,smpl2s):
        CBC.NormByMonCounts(smpl2, smpl1, "Lamb", norm_range, 1.0)

    drct1 = HBC.GetMonHistSAS(DrctRunNo, monUpStream, AxType, XRange, 0, DrctTSlice, frameBoundary, CalibEffi)
    drct2 = HBC.GetMonHistSAS(DrctRunNo, 2, AxType, XRange, 0, DrctTSlice, frameBoundary, CalibEffi)
    CBC.NormByMonCounts(drct2, drct1, "Lamb", norm_range, 1.0)

    ret = []
    for smpl2 in smpl2s:
        ret.append(Com.ExecFunctionCode("A/B", smpl2, drct2))
    if len(ret)==1:
        return ret[0]
    return ret



#!/usr/bin/python3
# -*- coding: utf-8 -*-

import utsusemi.SAS.ana.Reduction.HistBaseCommands as HBC

# reserved words for return value in this.
DAT = None
ecs = None
EC = None
ret = None
PyObj = None
Iq = None
Tr = None

SASBankDic={"SD":0,"SU":1,"SM16":2,"SM14":3,"SC":4,"MD":5,"MU":6,"ML":7,"MR":8,"HD":9,"HU":10,"HL":11,"BW":13}
########################################
def GetHistogramSAS( runNo=0, Xrange="0.7 7.6 0.1 dL", Bank="SM", TimeSlice="-1 -1", frameBoundary=0.0, isTimeFoc=False, SolAngCor=True, paramFiles="- -" ):
    """
    Load EventData to produce histogram
    GetHistogramSAS( runNo=0, AxType="lambda", Xrange="0.7 7.7 0.05", Bank="SM", TimeSlice="-1 -1", frameBoundary=0.0, isTimeFoc=False )
    
    @param runNo     (int)     Run Number
    @param Xrange    (string)  "startX endX widthX type" separated with space (not comma) : type=[dL, dL/L, tof]
    @param Bank      (string)  "SM"...  Monitors by "MON1" or "MON2"
    @param TimeSlice (string)  "startTime endTime" separated with space: "-1 -1" means no-setting.
    @param frameBoundary (float) 
    @param isTimeFoc (bool)    True or False
    @param SolAngCor  (bool)    True or False for Solid Angle Correction
    @param paramFiles (string) "WiringInfo DetectorInfo"
    @retval DAT (ElementContainerMatrix)
    """
    return HBC.GetHistogramSAS( runNo, Xrange, Bank, TimeSlice, str(frameBoundary), isTimeFoc, SolAngCor, paramFiles )

########################################
def GetMonHistSAS( runNo=0, MonNo=1, AxType="lambda", Xrange="0.7 7.6 0.1", MonIndex=0, TimeSlice="-1 -1", frameBoundary=0.0, CalibEffi=True ):
    """
    Load EventData to produce histogram of monitors
    GetMonHistSAS( runNo=0, MonNo=1, AxType="lambda", Xrange="0.7 7.7 0.05", MonIndex=0, TimeSlice="-1 -1", frameBoundary=0.0, CalibEffi=True )
    
    @param runNo     (int)     Run Number
    @param MonNo     (int)     Monitor Number 1 (up-stream) or 2 (down-stream)
    @param AxType    (string)  Type must be "tof', "lambda" or "lamda2"
    @param Xrange    (string)  "startX endX widthX" separated with space (not comma)
    @param MonIndex  (int)     
    @param TimeSlice (string)  "startTime endTime" separated with space: "-1 -1" means no-setting.
    @param frameBoundary (float) 
    @param CalibEffi (bool)    Monitor-detector efficiency calibration
    @retval EC (ElementContainer)
    """
    return HBC.GetMonHistSAS( runNo, MonNo, AxType, Xrange, MonIndex, TimeSlice, str(frameBoundary), CalibEffi )

########################################
def GetPulseHeightSAS( runNo=0, Xrange="0 5000 10", Bank="SM", TimeSlice="-1 -1", paramFiles="- -" ):
    """
    Make Pulse Height data of EventData
    GetPulseHeightSAS( runNo=0, Xrange="0 10000 10", Bank="SM", TimeSlice="-1 -1", paramFiles="- -" ):
    
    @param runNo     (int)     Run Number
    @param Xrange    (string)  "startX endX widthX type" separated with space (not comma) : type=[dL, dL/L, tof]
    @param Bank      (string)  "SM"...  Monitors by "MON1" or "MON2"
    @param TimeSlice (string)  "startTime endTime" separated with space: "-1 -1" means no-setting.
    @param paramFiles (string) "WiringInfo DetectorInfo"
    @retval ECM (ElementContainerMatrix)
    """
    frameBoundary="0.0"
    PHbin=5
    return HBC.GetPulseHeightSAS( runNo, Xrange, Bank,TimeSlice, frameBoundary, paramFiles, PHbin)

########################################
def GetMonHistByPhSAS( runNo=0, MonNo=1, Xrange="0.0 40000.0 10.0 tof", phRange="0 10000", MonIndex=0, TimeSlice="-1 -1", frameBoundary=0.0, CalibEffi=True ):
    """
    Load EventData to produce histogram of monitors limited by pulse height
    GetMonHistByPhSAS( runNo=0, MonNo=1, Xrange="0.0 40000.0 10.0 tof", phRange="0 10000", MonIndex=0, TimeSlice="-1 -1", frameBoundary=0.0, CalibEffi=True )
    
    @param runNo     (int)     Run Number
    @param MonNo     (int)     Monitor Number 1 (up-stream) or 2 (down-stream)
    @param Xrange    (string)  "startX endX widthX type" separated with space (not comma) : type=[dL, dL/L, tof]
    @param phRange   (string)  "lld hld" 
    @param MonIndex  (int)     0.. digital  1.. analog
    @param TimeSlice (string)  "startTime endTime" separated with space: "-1 -1" means no-setting.
    @param frameBoundary (float) 
    @param CalibEffi (bool)    Monitor-detector efficiency calibration
    @retval EC (ElementContainer)
    """
    return HBC.GetMonHistByPhSAS( runNo, MonNo, Xrange, phRange, MonIndex, TimeSlice, frameBoundary, CalibEffi )

########################################
# Dictionary for entry the name of functions
_functions = {
            "GetHistogramSAS":GetHistogramSAS,
            "GetMonHistSAS":GetMonHistSAS,
            "GetPulseHeightSAS":GetPulseHeightSAS,
            "GetMonHistByPhSAS":GetMonHistByPhSAS
            }
_functionsOrder = [
    "GetHistogramSAS",
    "GetMonHistSAS",
    "GetPulseHeightSAS",
    "GetMonHistByPhSAS"
    ]
    



#!/usr/bin/python
# -*- coding: utf-8 -*-
"""
PlaneSlicer.py
平面スライス処理
"""
from __future__ import print_function
import Manyo
import Manyo.MLF as mm
import time

#########################################
def mvec(values):
    """
    タプルデータをManyo のDoubleVectorに変換
    @param  values 実数のタプルデータ
    @retval Manyo Doblu Vector
    """
    vec = Manyo.MakeDoubleVector(len(values))
    i = 0
    for value in values:
        vec[i] = value
        i += 1
    return vec

#######################################
#  PlaneSlicer
#######################################
class PlaneSlicer(object) :
    """
    ElementContainerArray または Matrix の平面切り取り
    """
    #########################################
    def __init__(self,data = None):
        """
        コンストラクタ
        @param  data  エレメントコンテナアレイまたはマトリックスのポインタ
        @retval 無し
        """
        # データが指定されているか
        if data == None:
            self._pls = mm.MlfMatrixSlicer()
        else:
            self._pls = mm.MlfMatrixSlicer(data)
        self._org = None
        self._x = None
        self._y = None
        self._sliceCondition = None
        self._lines = Manyo.ElementContainerArray()
        self._start = None
        self._end = None
        self._lineCondition = None

    #########################################
    def SetAxes(self, keyX, keyY, keyZ):
        """
        3軸を、キーによって指定
        @param  keyX (string) X軸のキー
        @param  keyY (string) Y軸のキー
        @param  keyZ (string) Z軸のキー
        @retval True: OK, False : NG
        """
        if keyX != None and not isinstance(keyX, str):
            print("keyX must be string.")
            return False
        if keyY != None and not isinstance(keyY, str):
            print("keyY must be string.")
            return False
        if keyY != None and not isinstance(keyZ, str):
            print("keyZ must be string.")
            return False
        return self._pls.SetAxes(keyX, keyY, keyZ)

    #########################################
    def IsValid(self):
        """
        平面スライスが可能か
        @param  無し
        @retval True: Valid, False : Invalid
        """
        return self._pls.IsValid()

    #########################################
    def SetPlaneKeys(self, xkey, ykey):
        """
        平面のX軸とY軸ののKey文字列設定
        @param  xkey (string) 平面のX軸を示すキー
        @param  ykey (string) 平面のY軸を示すキー
        @retval 無し
        """
        if xkey != None and not isinstance(xkey, str):
            print("xkey is not string. Unable to set.")
            return
        if ykey != None and not isinstance(ykey, str):
            print("ykey is not string. Unable to set.")
            return
        self._pls.SetPlaneKeys(xkey, ykey)

    #########################################
    def SetPlane(self, plane):
        """
        平面の設定
        @param  plane (ElementContainerArray*) 平面
        @retval 無し
        """
        self._org = None
        self._x = None
        self._y = None
        self._sliceCondition = None
        self._lines = Manyo.ElementContainerArray()
        self._start = None
        self._end = None
        self._lineCondition = None
        return self._pls.SetPlane(plane)

    #########################################
    def GetPlane(self, org, x, y, xbin, ybin,Xrange,Yrange, thickness):
        """
        平面スライス
        @param  org (float,float,float) 原点を示す座標
        @param  x (float,float,float) X軸上の点の座標
        @param  y (float,float,float) Y軸上の点の座標
        @param  xbin (float) X軸のbin幅
        @param  ybin (float) Y軸のbin幅
        @param  rangex (float,float) X軸の範囲
        @param  rangey (float,float) Y軸の範囲
        @param  thickness (float) 平面の厚さ
        @retval ElementContainerArray
        """
        # 条件を保存
        self._org = [org[0],org[1],org[2]]
        self._x = [x[0],x[1],x[2]]
        self._y = [y[0],y[1],y[2]]
        self._sliceCondition = (xbin, ybin,Xrange,Yrange, thickness)
        # ラインは初期化
        del self._lines
        self._lines = Manyo.ElementContainerArray()
        self._start = None
        self._end = None
        self._lineCondition = None
        #t1 = time.time()
        # 平面の切り取り
        plane = self._pls.GetPlane(mvec(org), mvec(x), mvec(y), xbin, ybin, \
                                  mvec(Xrange), mvec(Yrange), thickness)
        #t2 = time.time()
        #print "Slice time(s): ", (t2-t1)
        return plane

    #########################################
    def GetNextPlane(self, xdiff, ydiff, zdiff):
        """
        次の平面をスライス
        @param  xdiff (float) 原点のシフト量(x)
        @param  ydiff (float) 原点のシフト量(y)
        @param  zdiff (float) 原点のシフト量(z)
        @retval ElementContainerArray
        """
        # 先に GetPlane が実行されているか
        if self._org == None:
            print("Execute 'GetPlane' before. ")
            return Manyo.ElementContainerArray()

        xbin, ybin,Xrange,Yrange, thickness = self._sliceCondition
        # 原点のシフト
        self._org[0] += xdiff
        self._org[1] += ydiff
        self._org[2] += zdiff
        # X軸上の点のシフト
        self._x[0] += xdiff
        self._x[1] += ydiff
        self._x[2] += zdiff
        # Y軸上の点のシフト
        self._y[0] += xdiff
        self._y[1] += ydiff
        self._y[2] += zdiff

        # シフト後の平面の切り取り
        return self._pls.GetPlane(mvec(self._org), mvec(self._x), mvec(self._y), xbin, ybin, \
                                  mvec(Xrange), mvec(Yrange), thickness)


    #########################################
    def SetLineKeys(self, xkey, ykey):
        """
        線分のX軸とY軸ののKey文字列設定
        @param  xkey (string) 平面のX軸を示すキー
        @param  ykey (string) 平面のY軸を示すキー
        @retval 無し
        """
        if xkey != None and not isinstance(xkey, str):
            print("xkey is not string. Unable to set.")
            return
        if ykey != None and not isinstance(ykey, str):
            print("ykey is not string. Unable to set.")
            return
        self._pls.SetLineKeys(xkey, ykey)

    #########################################
    def GetLine(self, start, end, lbin, width):
        """
        直線を切り出す
        @param  start (float, float) 始点(x,y)
        @param  end (float,float) 終点(x,y)
        @param  lbin (float) 刻み
        @param  width (float) 幅
        @retval ElementContainerArray
        """
        self._start = [start[0], start[1]]
        self._end = [end[0], end[1]]
        self._lineCondition = (lbin, width)
        line = self._pls.GetLine(mvec(start), mvec(end), lbin, width)
        # ラインが取得できていれば
        if line.PutSize() > 0:
            self._lines.Add(line)
        else:
            print("Failed to get line.")
        return line


    #########################################
    def GetLineInfo(self):
        """
        直線情報取得
        @param  無し
        @retval (start, end, (bin, width) )
        """
        return (self._start,self._end,self._lineCondition)

    #########################################
    def GetNextLine(self, xdiff, ydiff):
        """
        次の線分を取得
        @param  xdiff (float) 線分のシフト量(x)
        @param  ydiff (float) 線分のシフト量(y)
        @retval ElementContainer
        """
        # 先に GetPlane が実行されているか
        if self._start == None:
            print("Execute 'GetLine' before. ")
            return Manyo.ElementContainer()
        lbin, width = self._lineCondition
        # 始点のシフト
        self._start[0] += xdiff
        self._start[1] += ydiff
        # 終点のシフト
        self._end[0] += xdiff
        self._end[1] += ydiff
        line = self._pls.GetLine(mvec(self._start), mvec(self._end), lbin, width)
        # ラインが取得できていれば
        if line.PutSize() > 0:
            self._lines.Add(line)
        else:
            print("Failed to get line.")
        return line

    #########################################
    def ClearLines(self):
        """
        線分をクリア
        @param  無し
        @retval 無し
        """
        self._lines = Manyo.ElementContainerArray()

    #########################################
    def GetLines(self):
        """
        線分の集合を取得
        @param  無し
        @retval ElementContainerArray
        """
        return self._lines

